==> ideaIC-173.3531.6-sources.jar: com/intellij/codeInspection/VariableTypeCanBeExplicitInspection.java <==
// Copyright 2000-2017 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.codeInspection;

import com.intellij.codeHighlighting.HighlightDisplayLevel;
import com.intellij.openapi.project.Project;
import com.intellij.pom.java.LanguageLevel;
import com.intellij.psi.*;
import com.intellij.psi.codeStyle.CodeStyleManager;
import com.intellij.psi.codeStyle.JavaCodeStyleManager;
import com.intellij.psi.util.PsiUtil;
import com.intellij.util.IncorrectOperationException;
import com.siyeh.InspectionGadgetsBundle;
import org.jetbrains.annotations.Nls;
import org.jetbrains.annotations.NotNull;

public class VariableTypeCanBeExplicitInspection extends AbstractBaseJavaLocalInspectionTool {
  @NotNull
  @Override
  public PsiElementVisitor buildVisitor(@NotNull ProblemsHolder holder, boolean isOnTheFly) {
    if (!PsiUtil.getLanguageLevel(holder.getFile()).isAtLeast(LanguageLevel.JDK_X)) {
      return PsiElementVisitor.EMPTY_VISITOR;
    }
    PsiElementFactory elementFactory = JavaPsiFacade.getElementFactory(holder.getProject());
    return new JavaElementVisitor() {
      @Override
      public void visitVariable(PsiVariable variable) {
        PsiTypeElement typeElement = variable.getTypeElement();
        if (typeElement != null && typeElement.isInferredType()) {
          PsiType type = variable.getType();
          try {
            PsiType typeAfterReplacement = elementFactory.createTypeElementFromText(type.getCanonicalText(), variable).getType();
            if (type.equals(typeAfterReplacement)) {
              holder.registerProblem(typeElement,
                                     "'var' can be replaced with explicit type",
                                     ProblemHighlightType.INFORMATION,
                                     new ReplaceVarWithExplicitTypeFix());
            }
          }
          catch (IncorrectOperationException e) {
            //non-denotable type
          }
        }
      }
    };
  }

  @NotNull
  @Override
  public String getShortName() {
    return "VariableTypeCanBeExplicit";
  }

  @Nls
  @NotNull
  @Override
  public String getDisplayName() {
    return InspectionGadgetsBundle.message("variable.type.can.be.explicit.display.name");
  }

  @Nls
  @NotNull
  @Override
  public String getGroupDisplayName() {
    return "Java Experimental";
  }

  @NotNull
  @Override
  public String[] getGroupPath() {
    return new String[] {"Java", "Java language level migration aids", "Java Experimental"};
  }

  @NotNull
  @Override
  public HighlightDisplayLevel getDefaultLevel() {
    return HighlightDisplayLevel.WARNING;
  }

  @Override
  public boolean isEnabledByDefault() {
    return true;
  }

  private static class ReplaceVarWithExplicitTypeFix implements LocalQuickFix {
    @Nls
    @NotNull
    @Override
    public String getFamilyName() {
      return "Replace 'var' with explicit type";
    }

    @Override
    public void applyFix(@NotNull Project project, @NotNull ProblemDescriptor descriptor) {
      PsiElement element = descriptor.getPsiElement();
      if (element instanceof PsiTypeElement) {
        PsiTypeElement typeElementByExplicitType = JavaPsiFacade.getElementFactory(project)
          .createTypeElement(((PsiTypeElement)element).getType());
        PsiElement explicitTypeElement = element.replace(typeElementByExplicitType);
        explicitTypeElement = JavaCodeStyleManager.getInstance(project).shortenClassReferences(explicitTypeElement);
        CodeStyleManager.getInstance(project).reformat(explicitTypeElement);
      }
    }
  }
}

==> ideaIC-182.3208.16-sources.jar: com/intellij/codeInspection/VariableTypeCanBeExplicitInspection.java <==
// Copyright 2000-2018 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.codeInspection;

import com.intellij.openapi.project.Project;
import com.intellij.psi.*;
import com.intellij.psi.util.PsiTypesUtil;
import com.intellij.psi.util.PsiUtil;
import org.jetbrains.annotations.Nls;
import org.jetbrains.annotations.NotNull;

public class VariableTypeCanBeExplicitInspection extends AbstractBaseJavaLocalInspectionTool {
  @NotNull
  @Override
  public PsiElementVisitor buildVisitor(@NotNull ProblemsHolder holder, boolean isOnTheFly) {
    if (!PsiUtil.isLanguageLevel10OrHigher(holder.getFile())) { //var won't be parsed as inferred type otherwise
      return PsiElementVisitor.EMPTY_VISITOR;
    }
    return new JavaElementVisitor() {
      @Override
      public void visitVariable(PsiVariable variable) {
        PsiTypeElement typeElement = variable.getTypeElement();
        if (typeElement != null && typeElement.isInferredType()) {
          PsiType type = variable.getType();
          if (PsiTypesUtil.isDenotableType(type, variable)) {
            holder.registerProblem(typeElement,
                                   "'var' can be replaced with explicit type",
                                   ProblemHighlightType.GENERIC_ERROR_OR_WARNING,
                                   new ReplaceVarWithExplicitTypeFix());
          }
        }
      }
    };
  }

  private static class ReplaceVarWithExplicitTypeFix implements LocalQuickFix {
    @Nls
    @NotNull
    @Override
    public String getFamilyName() {
      return "Replace 'var' with explicit type";
    }

    @Override
    public void applyFix(@NotNull Project project, @NotNull ProblemDescriptor descriptor) {
      PsiElement element = descriptor.getPsiElement();
      if (element instanceof PsiTypeElement) {
        PsiTypesUtil.replaceWithExplicitType((PsiTypeElement)element);
      }
    }
  }
}
